@@ -246,7 +246,7 @@ void ArrowIpcDecoderReset(struct ArrowIpcDecoder* decoder) {
     ArrowArrayViewReset(&private_data->array_view);
 
     if (private_data->array.release != NULL) {
-      private_data->array.release(&private_data->array);
+      ArrowArrayRelease(&private_data->array);
     }
 
     if (private_data->fields != NULL) {
@@ -1117,21 +1117,24 @@ ArrowErrorCode ArrowIpcDecoderDecodeSchema(struct ArrowIpcDecoder* decoder,
   ArrowSchemaInit(&tmp);
   int result = ArrowSchemaSetTypeStruct(&tmp, n_fields);
   if (result != NANOARROW_OK) {
-    tmp.release(&tmp);
+    ArrowSchemaRelease(&tmp);
     ArrowErrorSet(error, "Failed to allocate struct schema with %ld children",
                   (long)n_fields);
     return result;
   }
 
+  // Top-level batch schema is typically non-nullable
+  tmp.flags = 0;
+
   result = ArrowIpcDecoderSetChildren(&tmp, fields, error);
   if (result != NANOARROW_OK) {
-    tmp.release(&tmp);
+    ArrowSchemaRelease(&tmp);
     return result;
   }
 
   result = ArrowIpcDecoderSetMetadata(&tmp, ns(Schema_custom_metadata(schema)), error);
   if (result != NANOARROW_OK) {
-    tmp.release(&tmp);
+    ArrowSchemaRelease(&tmp);
     return result;
   }
 
@@ -1155,7 +1158,7 @@ static void ArrowIpcDecoderInitFields(struct ArrowIpcField* fields,
   field->array = array;
   field->buffer_offset = *n_buffers;
 
-  for (int i = 0; i < 3; i++) {
+  for (int i = 0; i < NANOARROW_MAX_FIXED_BUFFERS; i++) {
     *n_buffers += array_view->layout.buffer_type[i] != NANOARROW_BUFFER_TYPE_NONE;
   }
 
@@ -1178,7 +1181,7 @@ ArrowErrorCode ArrowIpcDecoderSetSchema(struct ArrowIpcDecoder* decoder,
   private_data->n_fields = 0;
   ArrowArrayViewReset(&private_data->array_view);
   if (private_data->array.release != NULL) {
-    private_data->array.release(&private_data->array);
+    ArrowArrayRelease(&private_data->array);
   }
   if (private_data->fields != NULL) {
     ArrowFree(private_data->fields);
@@ -1524,7 +1527,7 @@ static int ArrowIpcDecoderWalkSetArrayView(struct ArrowIpcArraySetter* setter,
   array_view->null_count = ns(FieldNode_null_count(field));
   setter->field_i += 1;
 
-  for (int64_t i = 0; i < 3; i++) {
+  for (int i = 0; i < NANOARROW_MAX_FIXED_BUFFERS; i++) {
     if (array_view->layout.buffer_type[i] == NANOARROW_BUFFER_TYPE_NONE) {
       break;
     }
@@ -1675,7 +1678,7 @@ ArrowErrorCode ArrowIpcDecoderDecodeArray(struct ArrowIpcDecoder* decoder,
   int result =
       ArrowIpcDecoderDecodeArrayInternal(decoder, i, &temp, validation_level, error);
   if (result != NANOARROW_OK && temp.release != NULL) {
-    temp.release(&temp);
+    ArrowArrayRelease(&temp);
   } else if (result != NANOARROW_OK) {
     return result;
   }
@@ -1699,7 +1702,7 @@ ArrowErrorCode ArrowIpcDecoderDecodeArrayFromShared(
   int result =
       ArrowIpcDecoderDecodeArrayInternal(decoder, i, &temp, validation_level, error);
   if (result != NANOARROW_OK && temp.release != NULL) {
-    temp.release(&temp);
+    ArrowArrayRelease(&temp);
   } else if (result != NANOARROW_OK) {
     return result;
   }
