@@ -33,7 +33,7 @@ TEST(NanoarrowHppTest, NanoarrowHppUniqueArrayTest) {
   nanoarrow::UniqueArray array;
   EXPECT_EQ(array->release, nullptr);
 
-  ArrowArrayInitFromType(array.get(), NANOARROW_TYPE_INT32);
+  ASSERT_EQ(ArrowArrayInitFromType(array.get(), NANOARROW_TYPE_INT32), NANOARROW_OK);
   ASSERT_EQ(ArrowArrayStartAppending(array.get()), NANOARROW_OK);
   ASSERT_EQ(ArrowArrayAppendInt(array.get(), 123), NANOARROW_OK);
   ASSERT_EQ(ArrowArrayFinishBuildingDefault(array.get(), nullptr), NANOARROW_OK);
@@ -58,7 +58,7 @@ TEST(NanoarrowHppTest, NanoarrowHppUniqueSchemaTest) {
   nanoarrow::UniqueSchema schema;
   EXPECT_EQ(schema->release, nullptr);
 
-  ArrowSchemaInitFromType(schema.get(), NANOARROW_TYPE_INT32);
+  ASSERT_EQ(ArrowSchemaInitFromType(schema.get(), NANOARROW_TYPE_INT32), NANOARROW_OK);
   EXPECT_NE(schema->release, nullptr);
   EXPECT_STREQ(schema->format, "i");
 
@@ -86,7 +86,8 @@ TEST(NanoarrowHppTest, NanoarrowHppUniqueArrayStreamTest) {
   EXPECT_EQ(ArrowSchemaInitFromType(schema_in.get(), NANOARROW_TYPE_INT32), NANOARROW_OK);
   auto array_stream = nanoarrow::EmptyArrayStream::MakeUnique(schema_in.get());
   EXPECT_NE(array_stream->release, nullptr);
-  EXPECT_EQ(array_stream->get_schema(array_stream.get(), schema.get()), NANOARROW_OK);
+  EXPECT_EQ(ArrowArrayStreamGetSchema(array_stream.get(), schema.get(), nullptr),
+            NANOARROW_OK);
   EXPECT_STREQ(schema->format, "i");
   schema.reset();
   schema->format = NULL;
@@ -95,7 +96,8 @@ TEST(NanoarrowHppTest, NanoarrowHppUniqueArrayStreamTest) {
   nanoarrow::UniqueArrayStream array_stream2 = std::move(array_stream);
   EXPECT_EQ(array_stream->release, nullptr);
   EXPECT_NE(array_stream2->release, nullptr);
-  EXPECT_EQ(array_stream2->get_schema(array_stream2.get(), schema.get()), NANOARROW_OK);
+  EXPECT_EQ(ArrowArrayStreamGetSchema(array_stream2.get(), schema.get(), nullptr),
+            NANOARROW_OK);
   EXPECT_STREQ(schema->format, "i");
   schema.reset();
   schema->format = NULL;
@@ -104,7 +106,8 @@ TEST(NanoarrowHppTest, NanoarrowHppUniqueArrayStreamTest) {
   nanoarrow::UniqueArrayStream array_stream3(array_stream2.get());
   EXPECT_EQ(array_stream2->release, nullptr);
   EXPECT_NE(array_stream3->release, nullptr);
-  EXPECT_EQ(array_stream3->get_schema(array_stream2.get(), schema.get()), NANOARROW_OK);
+  EXPECT_EQ(ArrowArrayStreamGetSchema(array_stream2.get(), schema.get(), nullptr),
+            NANOARROW_OK);
   EXPECT_STREQ(schema->format, "i");
 
   // releasing should clear the release callback
@@ -171,7 +174,7 @@ TEST(NanoarrowHppTest, NanoarrowHppUniqueArrayViewTest) {
   // Use an ArrayView with children, since an ArrayView with no children
   // doesn't hold any resources
   ArrowArrayViewInitFromType(array_view.get(), NANOARROW_TYPE_STRUCT);
-  ArrowArrayViewAllocateChildren(array_view.get(), 2);
+  ASSERT_EQ(ArrowArrayViewAllocateChildren(array_view.get(), 2), NANOARROW_OK);
   EXPECT_EQ(array_view->storage_type, NANOARROW_TYPE_STRUCT);
 
   // move constructor
@@ -191,20 +194,19 @@ TEST(NanoarrowHppTest, NanoarrowHppEmptyArrayStreamTest) {
 
   nanoarrow::UniqueSchema schema_in;
   EXPECT_EQ(ArrowSchemaInitFromType(schema_in.get(), NANOARROW_TYPE_INT32), NANOARROW_OK);
-  auto array_stream = nanoarrow::EmptyArrayStream::MakeUnique(schema_in.get());
 
-  EXPECT_EQ(array_stream->get_schema(array_stream.get(), schema.get()), NANOARROW_OK);
+  nanoarrow::UniqueArrayStream array_stream;
+  nanoarrow::EmptyArrayStream(schema_in.get()).ToArrayStream(array_stream.get());
+
+  EXPECT_EQ(ArrowArrayStreamGetSchema(array_stream.get(), schema.get(), nullptr),
+            NANOARROW_OK);
   EXPECT_STREQ(schema->format, "i");
-  EXPECT_EQ(array_stream->get_next(array_stream.get(), &array), NANOARROW_OK);
+  EXPECT_EQ(ArrowArrayStreamGetNext(array_stream.get(), &array, nullptr), NANOARROW_OK);
   EXPECT_EQ(array.release, nullptr);
-  EXPECT_STREQ(array_stream->get_last_error(array_stream.get()), "");
+  EXPECT_STREQ(ArrowArrayStreamGetLastError(array_stream.get()), "");
 }
 
 TEST(NanoarrowHppTest, NanoarrowHppVectorArrayStreamTest) {
-  nanoarrow::UniqueSchema schema;
-  nanoarrow::UniqueArray array;
-  nanoarrow::UniqueArrayView array_view;
-
   nanoarrow::UniqueArray array_in;
   EXPECT_EQ(ArrowArrayInitFromType(array_in.get(), NANOARROW_TYPE_INT32), NANOARROW_OK);
   EXPECT_EQ(ArrowArrayStartAppending(array_in.get()), NANOARROW_OK);
@@ -214,15 +216,28 @@ TEST(NanoarrowHppTest, NanoarrowHppVectorArrayStreamTest) {
   nanoarrow::UniqueSchema schema_in;
   EXPECT_EQ(ArrowSchemaInitFromType(schema_in.get(), NANOARROW_TYPE_INT32), NANOARROW_OK);
 
-  auto array_stream =
-      nanoarrow::VectorArrayStream::MakeUnique(schema_in.get(), array_in.get());
+  nanoarrow::UniqueArrayStream array_stream;
+  nanoarrow::VectorArrayStream(schema_in.get(), array_in.get())
+      .ToArrayStream(array_stream.get());
+
+  nanoarrow::UniqueSchema schema;
+  ASSERT_EQ(array_stream->get_schema(array_stream.get(), schema.get()), NANOARROW_OK);
+
+  nanoarrow::UniqueArrayView array_view;
+  ASSERT_EQ(ArrowArrayViewInitFromSchema(array_view.get(), schema.get(), nullptr),
+            NANOARROW_OK);
+
+  nanoarrow::UniqueArray array;
+  EXPECT_EQ(ArrowArrayStreamGetNext(array_stream.get(), array.get(), nullptr),
+            NANOARROW_OK);
 
-  EXPECT_EQ(array_stream->get_next(array_stream.get(), array.get()), NANOARROW_OK);
-  ArrowArrayViewInitFromType(array_view.get(), NANOARROW_TYPE_INT32);
   ASSERT_EQ(ArrowArrayViewSetArray(array_view.get(), array.get(), nullptr), NANOARROW_OK);
   EXPECT_EQ(ArrowArrayViewGetIntUnsafe(array_view.get(), 0), 1234);
   array.reset();
 
-  EXPECT_EQ(array_stream->get_next(array_stream.get(), array.get()), NANOARROW_OK);
+  EXPECT_EQ(ArrowArrayStreamGetNext(array_stream.get(), array.get(), nullptr),
+            NANOARROW_OK);
   EXPECT_EQ(array->release, nullptr);
+
+  EXPECT_STREQ(array_stream->get_last_error(array_stream.get()), "");
 }
